package Asamasi::RakuAPI;

use strict;
use warnings;

use LWP::Simple;
use Cache::File;
use URI::Escape;

require Exporter;

our $VERSION = '0.1';

# params: cache, affiliate_code
sub new {
    my $class = shift;
    my $self = {};
    my %param = @_;
    for (keys %param) { $self->{lc($_)} = $param{$_} } 
    return bless $self, $class;
}

# params: key, affiliate_code, row, genre, sort
sub search {
    my ($self, $args_ref) = @_;
    my $key = URI::Escape::uri_escape($args_ref->{key});
    my Cache::File $cache = $self->{cache};
    my $acode = $args_ref->{affiliate_code} || $self->{affiliate_code};
    my $row = $args_ref->{row};
    my $genre = $args_ref->{genre};
    my $sort = $args_ref->{sort};

    my $url = "http://rakuapi.ddo.jp/api"
	."?keyword=$key"
	."&encode=utf-8"
	."&output=tsv"
	.($row ? "&row=$row" : "")
	.($genre ? "&genre=$genre" : "")
	.($sort ? "&sort=$sort" : "")
	.($acode ? "&affiliate_code=$acode" : "")
	;

    my $data;
    if (defined $cache) {
	$data = $cache->get($url);
	if (not $data) {
	    $data = LWP::Simple::get($url);
	    $cache->set($url, $data);
	}
    } else {
	$data = LWP::Simple::get($url);
    }

    my @results;
    $data =~ s/^.+?\n//;
    while ($data =~ m!^(.+)$!gm) {
	my @cols = split(/\t/, $1);
	push @results, {
	    Title => $cols[0],
	    Url => $cols[2],
	    ThumbnailUrl => $cols[3], 
	    Price => $cols[5],
	    Tax => $cols[6],
	    ShopName => $cols[8],
	    ShopUrl => $cols[9],
	}
    }

    return \@results;
}


1;
__END__

=head1 NAME

Asamasi::RakuAPI - Handle RakuAPI with Perl

=head1 SYNOPSIS

  use Asamasi::RakuAPI;

  my $ar = Asamasi::RakuAPI->new(affiliate_code => $acode);

  my $results_ref = $ar->search({key => $key, row => 3});

  for (@{$results_ref}) {
    print $_->{Title}, $_->{Url}, ...
  }

  # Responses can be cached locally by "Cache::File"
  # (can be downloaded from CPAN).

  use Cache::File;

  my $cache = Cache::File->new(cache_root => "/var/tmp/cache",
			       default_expires => '48 hours');

  my $ar = Asamasi::RakuAPI->new(cache => $cache, 
				 affiliate_code => $acode);

=head1 DESCRIPTION

RakuAPI (http://rakuapi.ddo.jp/)

=head2 FUNCTION

new - constructor.

  params: cache, affiliate_code

search -  Returns search result (reference of array of hash).

  params: key, affiliate_code, row, genre, sort
  returns: Title, Url, ThumbnailUrl, Price, Tax, ShopName, ShopUrl

=head1 AUTHOR

Tatsuo Yamashita, E<lt>yto at nais.toE<gt>


=cut
